// Generated by CoffeeScript 1.6.3
(function() {
  var _mongoOpsFromTo, _processRevisions, _updateManyOptions, _updateOneOptions, _updateOpInstruction, _updateOpRollback,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __slice = [].slice;

  _processRevisions = function(instructionName, config, collectionName, revisions) {
    var key, revisionName, revisionNames, revisionNumbers, _i, _len;
    if ((config.revisions == null) || (config.revisions[collectionName] == null)) {
      return [new Error("can't " + instructionName + " documents in " + collectionName + " without relevant config.revisions supplied at committed.start")];
    }
    if (revisions == null) {
      revisions = config.revisions[collectionName];
    }
    if (typeof revisions === 'string') {
      revisions = [revisions];
    }
    if (Array.isArray(revisions)) {
      revisionNames = revisions;
      revisionNumbers = null;
    } else {
      revisionNames = (function() {
        var _results;
        _results = [];
        for (key in revisions) {
          _results.push(key);
        }
        return _results;
      })();
      revisionNumbers = revisions;
    }
    for (_i = 0, _len = revisionNames.length; _i < _len; _i++) {
      revisionName = revisionNames[_i];
      if (__indexOf.call(config.revisions[collectionName], revisionName) < 0) {
        return [new Error("can't " + instructionName + " documents with a revision '" + revisionName + "' not specified in the config.revisions given at committed.start")];
      }
    }
    return [null, revisionNames, revisionNumbers];
  };

  exports.insert = {
    preSave: function(config, transaction, state, _arg, done) {
      var collectionName, documents, error, etc, revisionNames, revisionNumbers, revisions, _ref;
      collectionName = _arg[0], documents = _arg[1], revisions = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      if (etc.length !== 0) {
        return done(new Error("too many values passed to insert"));
      }
      if (!((collectionName != null) && (documents != null))) {
        return done(new Error("null or missing argument to insert command"));
      }
      if (!Array.isArray(documents)) {
        documents = [documents];
      }
      _ref = _processRevisions('insert', config, collectionName, revisions), error = _ref[0], revisionNames = _ref[1], revisionNumbers = _ref[2];
      if (error != null) {
        return done(error);
      }
      if (revisionNumbers != null) {
        return done(new Error("can't insert using specific revision numbers " + revisionNumbers));
      }
      if (documents.some(function(d) {
        return d.revision != null;
      })) {
        return done(new Error("can't insert documents when some already have a revision property"));
      }
      return config.db.collection(collectionName, {
        strict: true
      }, function(err, collection) {
        var d, revisionName, _i, _j, _len, _len1;
        if (err != null) {
          return done(err);
        }
        state.ids = [];
        for (_i = 0, _len = documents.length; _i < _len; _i++) {
          d = documents[_i];
          if (d._id != null) {
            state.ids.push(d._id);
          } else {
            d._id = _pkFactory.createPk();
            state.ids.push(d._id);
          }
          if (revisionNames.length > 0 && (d.revision == null)) {
            d.revision = {};
            for (_j = 0, _len1 = revisionNames.length; _j < _len1; _j++) {
              revisionName = revisionNames[_j];
              d.revision[revisionName] = 0;
            }
          }
        }
        return done();
      });
    },
    execute: function(config, transaction, state, _arg, done) {
      var collectionName, documents, etc, options, revisions;
      collectionName = _arg[0], documents = _arg[1], revisions = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      options = {
        w: 1,
        journal: true,
        serializeFunctions: false
      };
      return collection.insert(documents, options, function(err, objects) {
        if (err != null) {
          return done(err, null);
        }
        return done(null, true);
      });
    }
  };

  exports.insertRollback = {
    preSave: function(config, transaction, state, _arg, done) {
      var collectionName, documents, etc, revisions;
      collectionName = _arg[0], documents = _arg[1], revisions = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      if (etc.length !== 0) {
        return done(new Error("too many values passed to insertRollback"));
      }
      if (!((collectionName != null) && (documents != null))) {
        return done(new Error("null or missing argument to insertRollback command"));
      }
      return done();
    },
    execute: function(config, transaction, state, _arg, done) {
      var collectionName, documents, etc, revisions;
      collectionName = _arg[0], documents = _arg[1], revisions = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      return config.db.collection(collectionName, {
        strict: true
      }, function(err, collection) {
        var d, i, iterator, options, _i, _len;
        if (err != null) {
          return done(err, false);
        }
        options = {
          w: 1,
          journal: true,
          serializeFunctions: false
        };
        if (state.ids == null) {
          return done(null, true);
        } else {
          if (!Array.isArray(documents)) {
            documents = [documents];
          }
          if (documents.length !== state.ids.length) {
            return done(new Error("during rollback, number of documents to insert doesnt match the number of ids generated during insert"), false);
          }
          for (i = _i = 0, _len = documents.length; _i < _len; i = ++_i) {
            d = documents[i];
            if (d._id == null) {
              d._id = state.ids[i];
            }
          }
          iterator = function(item, itemDone) {
            return collection.remove(item, options, function(err, result) {
              return itemDone(err);
            });
          };
          return async.each(documents, iterator, function(err) {
            return done(err, true);
          });
        }
      });
    }
  };

  _updateOneOptions = {
    w: 1,
    journal: true,
    upsert: false,
    multi: false,
    serializeFunctions: false
  };

  _updateManyOptions = {
    w: 1,
    journal: true,
    upsert: false,
    multi: true,
    serializeFunctions: false
  };

  _mongoOpsFromTo = function(mongoOps, fromDoc, toDoc) {
    var fromFlattened, key, newDotPaths, oldDotPaths, path, toFlattened, unsetDotPaths, value, _i, _j, _len, _len1, _ref;
    toFlattened = _mongoFlatten(toDoc);
    fromFlattened = _mongoFlatten(fromDoc);
    for (_i = 0, _len = toFlattened.length; _i < _len; _i++) {
      _ref = toFlattened[_i], key = _ref[0], value = _ref[1];
      if (key.indexOf('revision.') !== 0) {
        mongoOps.$set[key] = value;
      }
    }
    newDotPaths = (function() {
      var _ref1, _results;
      _ref1 = mongoOps.$set;
      _results = [];
      for (key in _ref1) {
        value = _ref1[key];
        _results.push(key);
      }
      return _results;
    })();
    oldDotPaths = (function() {
      var _j, _len1, _ref1, _results;
      _results = [];
      for (_j = 0, _len1 = fromFlattened.length; _j < _len1; _j++) {
        _ref1 = fromFlattened[_j], key = _ref1[0], value = _ref1[1];
        if (key.indexOf('revision.') !== 0) {
          _results.push(key);
        }
      }
      return _results;
    })();
    unsetDotPaths = (function() {
      var _j, _len1, _results;
      _results = [];
      for (_j = 0, _len1 = oldDotPaths.length; _j < _len1; _j++) {
        path = oldDotPaths[_j];
        if (__indexOf.call(newDotPaths, path) < 0) {
          _results.push(path);
        }
      }
      return _results;
    })();
    for (_j = 0, _len1 = unsetDotPaths.length; _j < _len1; _j++) {
      path = unsetDotPaths[_j];
      mongoOps.$unset[path] = 1;
    }
    return delete mongoOps.$set._id;
  };

  _updateOpInstruction = function(onlyOne, config, transaction, state, collectionName, selector, updateOps, rollbackProjector, revisions, etc, done) {
    var error, instructionName, revisionNames, revisionNumbers, _ref;
    instructionName = onlyOne ? 'updateOneOp' : 'updateManyOp';
    if (etc.length !== 0) {
      return done(new Error("too many values passed to " + instructionName));
    }
    if (!((collectionName != null) && (selector != null) && (updateOps != null))) {
      return done(new Error("null or missing argument to " + instructionName + " instruction"));
    }
    _ref = _processRevisions(instructionName, config, collectionName, revisions), error = _ref[0], revisionNames = _ref[1], revisionNumbers = _ref[2];
    if (error != null) {
      return done(error);
    }
    if (!onlyOne && (revisionNumbers != null)) {
      return done(new Error("can't " + instructionName + " with a revisions object that specifies exact revision numbers, that's only possible for singe updates"));
    }
    if ((updateOps.__inc != null) && revisionNames.some(function(name) {
      return updateOps.__inc["revision." + name];
    })) {
      return done(new Error("can't " + instructionName + " with revisions " + revisionNames + " when there are already $inc operations on the revision sub-document"));
    }
    if ((revisionNumbers != null) && revisionNames.some(function(name) {
      return selector["revision." + name];
    })) {
      return done(new Error("can't " + instructionName + " with revisions " + revisionNumbers + " with a selector that already matches on the revision subdoc"));
    }
    return config.db.collection(collectionName, {
      strict: true
    }, function(err, collection) {
      var mongoProjector, mongoSelector;
      if (err != null) {
        return done(err, false);
      }
      mongoSelector = _mongolize(selector);
      if (rollbackProjector != null) {
        mongoProjector = _mongolize(rollbackProjector);
        mongoProjector._id = 1;
        mongoProjector.revision = 1;
      } else {
        mongoProjector = {};
      }
      return collection.find(mongoSelector, mongoProjector).toArray(function(err, docs) {
        if (err != null) {
          return done(err, null);
        }
        if (onlyOne && docs.length !== 1) {
          transaction.execution.info.push("" + instructionName + " can update only one document, using " + (JSON.stringify(mongoSelector)) + " " + docs.length + " were found");
          return done(null, false);
        }
        if (onlyOne) {
          state.updated = docs[0];
        } else {
          state.updated = docs;
        }
        return _updateTransactionState(transaction, function(err) {
          var executeUpdate, matches, revisionName, revisionNumber;
          if (err != null) {
            return done(err, null);
          }
          executeUpdate = function() {
            var mongoUpdateOps, options, revisionName, _i, _len;
            if (onlyOne) {
              options = _updateOneOptions;
            } else {
              options = _updateManyOptions;
            }
            mongoUpdateOps = _mongolize(updateOps);
            if (revisionNames.length > 0) {
              if (mongoUpdateOps.$inc == null) {
                mongoUpdateOps.$inc = {};
              }
              for (_i = 0, _len = revisionNames.length; _i < _len; _i++) {
                revisionName = revisionNames[_i];
                mongoUpdateOps.$inc["revision." + revisionName] = 1;
              }
            }
            return collection.update(mongoSelector, mongoUpdateOps, options, function(err, updated) {
              if (err != null) {
                return done(err, null);
              }
              if (onlyOne && updated !== 1) {
                transaction.execution.info.push("" + instructionName + " can update only one document, using " + (JSON.stringify(mongoSelector)) + " " + updated + " were updated");
                return done(null, false);
              }
              if (onlyOne) {
                return done(null, true);
              } else {
                return done(null, true, updated);
              }
            });
          };
          if ((revisionNumbers != null) && onlyOne) {
            matches = (function() {
              var _results;
              _results = [];
              for (revisionName in revisionNumbers) {
                revisionNumber = revisionNumbers[revisionName];
                _results.push(state.updated.revision[revisionName] === revisionNumber);
              }
              return _results;
            })();
            if (!matches.every(function(x) {
              return x;
            })) {
              transaction.execution.info.push("" + instructionName + " failed to find documents to match the required revisions. \nFound: " + (JSON.stringify(state.updated)) + ". \nRequired: " + (JSON.stringify(revisionNumbers)) + ".");
              return done(null, false);
            } else {
              return executeUpdate();
            }
          } else {
            return executeUpdate();
          }
        });
      });
    });
  };

  _updateOpRollback = function(onlyOne, config, transaction, state, collectionName, selector, updateOps, rollbackProjector, revisions, etc, done) {
    var error, instructionName, revisionNames, revisionNumbers, _ref;
    instructionName = onlyOne ? 'updateOneOpRollback' : 'updateManyOpRollback';
    if (etc.length !== 0) {
      return done(new Error("too many values passed to " + instructionName));
    }
    if (!((collectionName != null) && (selector != null) && (updateOps != null))) {
      return done(new Error("null or missing argument to " + instructionName + " command"));
    }
    _ref = _processRevisions(instructionName, config, collectionName, revisions), error = _ref[0], revisionNames = _ref[1], revisionNumbers = _ref[2];
    if (error != null) {
      return done(error);
    }
    return config.db.collection(collectionName, {
      strict: true
    }, function(err, collection) {
      var docs, rollbackDoc;
      if (err != null) {
        return done(err, false);
      }
      if (state.updated == null) {
        return done(null, true);
      } else {
        docs = onlyOne ? [state.updated] : state.updated;
        rollbackDoc = function(doc, rollbackDone) {
          var revisionName, revisionNumber, revisionSelector, _ref1;
          revisionSelector = {
            _id: doc._id
          };
          _ref1 = doc.revision;
          for (revisionName in _ref1) {
            revisionNumber = _ref1[revisionName];
            revisionSelector["revision." + revisionName] = revisionNumber;
          }
          return collection.count(revisionSelector, function(err, count) {
            var flattened, from, key, mongoRollbackOps, mongoSelector, op, originalDotPaths, path, renamedTo, settyOperators, to, unsetDotPaths, updatedDotPaths, value, _i, _j, _k, _l, _len, _len1, _len2, _len3, _len4, _m, _ref2;
            if (count === 1) {
              return rollbackDone(null, true);
            } else if (count === 0) {
              mongoRollbackOps = {
                $set: {},
                $inc: {},
                $unset: {}
              };
              for (_i = 0, _len = revisionNames.length; _i < _len; _i++) {
                revisionName = revisionNames[_i];
                mongoRollbackOps.$inc["revision." + revisionName] = -1;
              }
              flattened = _mongoFlatten(doc);
              for (_j = 0, _len1 = flattened.length; _j < _len1; _j++) {
                _ref2 = flattened[_j], key = _ref2[0], value = _ref2[1];
                if (key.indexOf('revision.') !== 0) {
                  mongoRollbackOps.$set[key] = value;
                }
              }
              originalDotPaths = (function() {
                var _ref3, _results;
                _ref3 = mongoRollbackOps.$set;
                _results = [];
                for (key in _ref3) {
                  value = _ref3[key];
                  if (key.indexOf('revision.') !== 0) {
                    _results.push(key);
                  }
                }
                return _results;
              })();
              settyOperators = ['__inc', '__setOnInsert', '__set', '__addToSet', '__push', '__bit'];
              updatedDotPaths = [];
              for (_k = 0, _len2 = settyOperators.length; _k < _len2; _k++) {
                op = settyOperators[_k];
                if (updateOps[op] != null) {
                  for (path in updateOps[op]) {
                    updatedDotPaths.push(path);
                  }
                }
              }
              unsetDotPaths = (function() {
                var _l, _len3, _results;
                _results = [];
                for (_l = 0, _len3 = updatedDotPaths.length; _l < _len3; _l++) {
                  path = updatedDotPaths[_l];
                  if (__indexOf.call(originalDotPaths, path) < 0) {
                    _results.push(path);
                  }
                }
                return _results;
              })();
              if (updateOps['__rename'] != null) {
                renamedTo = (function() {
                  var _ref3, _results;
                  _ref3 = updateOps['__rename'];
                  _results = [];
                  for (from in _ref3) {
                    to = _ref3[from];
                    _results.push(to);
                  }
                  return _results;
                })();
                for (_l = 0, _len3 = renamedTo.length; _l < _len3; _l++) {
                  to = renamedTo[_l];
                  if (__indexOf.call(originalDotPaths, to) < 0) {
                    unsetDotPaths.push(to);
                  }
                }
              }
              for (_m = 0, _len4 = unsetDotPaths.length; _m < _len4; _m++) {
                path = unsetDotPaths[_m];
                mongoRollbackOps.$unset[path] = 1;
              }
              delete mongoRollbackOps.$set._id;
              mongoSelector = {
                _id: doc._id
              };
              return collection.update(mongoSelector, mongoRollbackOps, _updateOneOptions, function(err, updated) {
                if (err != null) {
                  return rollbackDone(err, false);
                }
                if (updated !== 1) {
                  transaction.execution.info.push("updateOneOpRollback failed to update exactly one document (actually " + updated + ") during rollback");
                  return rollbackDone(null, false);
                }
                return rollbackDone(null, true);
              });
            } else {
              transaction.execution.info.push("updateOneOpRollback found too many documents (" + count + ") when trying to find out if an updateOneOp had occurred");
              return rollbackDone(null, false);
            }
          });
        };
        return async.mapSeries(docs, rollbackDoc, function(err, rollbackResults) {
          if (err != null) {
            return done(err, null);
          }
          return done(err, rollbackResults.every(function(x) {
            return x;
          }));
        });
      }
    });
  };

  exports.db = {
    pass: function(config, transaction, state, args, done) {
      return done(null, true);
    },
    insert: function(config, transaction, state, _arg, done) {
      var collectionName, documents, error, etc, revisionNames, revisionNumbers, revisions, _ref;
      collectionName = _arg[0], documents = _arg[1], revisions = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      if (etc.length !== 0) {
        return done(new Error("too many values passed to insert"));
      }
      if (!((collectionName != null) && (documents != null))) {
        return done(new Error("null or missing argument to insert command"));
      }
      if (!Array.isArray(documents)) {
        documents = [documents];
      }
      _ref = _processRevisions('insert', config, collectionName, revisions), error = _ref[0], revisionNames = _ref[1], revisionNumbers = _ref[2];
      if (error != null) {
        return done(error);
      }
      if (revisionNumbers != null) {
        return done(new Error("can't insert using specific revision numbers " + revisionNumbers));
      }
      if (documents.some(function(d) {
        return d.revision != null;
      })) {
        return done(new Error("can't insert documents when some already have a revision property"));
      }
      return config.db.collection(collectionName, {
        strict: true
      }, function(err, collection) {
        var d, options, revisionName, _i, _j, _len, _len1;
        if (err != null) {
          return done(err, false);
        }
        state.ids = [];
        for (_i = 0, _len = documents.length; _i < _len; _i++) {
          d = documents[_i];
          if (d._id != null) {
            state.ids.push(d._id);
          } else {
            d._id = _pkFactory.createPk();
            state.ids.push(d._id);
          }
          if (revisionNames.length > 0 && (d.revision == null)) {
            d.revision = {};
            for (_j = 0, _len1 = revisionNames.length; _j < _len1; _j++) {
              revisionName = revisionNames[_j];
              d.revision[revisionName] = 0;
            }
          }
        }
        options = {
          w: 1,
          journal: true,
          serializeFunctions: false
        };
        return _updateTransactionState(transaction, function(err) {
          if (err != null) {
            return done(err, null);
          }
          return collection.insert(documents, options, function(err, objects) {
            if (err != null) {
              return done(err, null);
            }
            return done(null, true);
          });
        });
      });
    },
    insertRollback: function(config, transaction, state, _arg, done) {
      var collectionName, documents, etc, revisions;
      collectionName = _arg[0], documents = _arg[1], revisions = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      if (etc.length !== 0) {
        return done(new Error("too many values passed to insertRollback"));
      }
      if (!((collectionName != null) && (documents != null))) {
        return done(new Error("null or missing argument to insertRollback command"));
      }
      return config.db.collection(collectionName, {
        strict: true
      }, function(err, collection) {
        var d, i, iterator, options, _i, _len;
        if (err != null) {
          return done(err, false);
        }
        options = {
          w: 1,
          journal: true,
          serializeFunctions: false
        };
        if (state.ids == null) {
          return done(null, true);
        } else {
          if (!Array.isArray(documents)) {
            documents = [documents];
          }
          if (documents.length !== state.ids.length) {
            return done(new Error("during rollback, number of documents to insert doesnt match the number of ids generated during insert"), false);
          }
          for (i = _i = 0, _len = documents.length; _i < _len; i = ++_i) {
            d = documents[i];
            if (d._id == null) {
              d._id = state.ids[i];
            }
          }
          iterator = function(item, itemDone) {
            return collection.remove(item, options, function(err, result) {
              return itemDone(err);
            });
          };
          return async.each(documents, iterator, function(err) {
            return done(err, true);
          });
        }
      });
    },
    updateOneOp: function(config, transaction, state, _arg, done) {
      var collectionName, etc, revisions, rollbackProjector, selector, updateOps;
      collectionName = _arg[0], selector = _arg[1], updateOps = _arg[2], rollbackProjector = _arg[3], revisions = _arg[4], etc = 6 <= _arg.length ? __slice.call(_arg, 5) : [];
      return _updateOpInstruction(true, config, transaction, state, collectionName, selector, updateOps, rollbackProjector, revisions, etc, done);
    },
    updateOneOpRollback: function(config, transaction, state, _arg, done) {
      var collectionName, etc, revisions, rollbackProjector, selector, updateOps;
      collectionName = _arg[0], selector = _arg[1], updateOps = _arg[2], rollbackProjector = _arg[3], revisions = _arg[4], etc = 6 <= _arg.length ? __slice.call(_arg, 5) : [];
      return _updateOpRollback(true, config, transaction, state, collectionName, selector, updateOps, rollbackProjector, revisions, etc, done);
    },
    updateManyOp: function(config, transaction, state, _arg, done) {
      var collectionName, etc, revisions, rollbackProjector, selector, updateOps;
      collectionName = _arg[0], selector = _arg[1], updateOps = _arg[2], rollbackProjector = _arg[3], revisions = _arg[4], etc = 6 <= _arg.length ? __slice.call(_arg, 5) : [];
      return _updateOpInstruction(false, config, transaction, state, collectionName, selector, updateOps, rollbackProjector, revisions, etc, done);
    },
    updateManyOpRollback: function(config, transaction, state, _arg, done) {
      var collectionName, etc, revisions, rollbackProjector, selector, updateOps;
      collectionName = _arg[0], selector = _arg[1], updateOps = _arg[2], rollbackProjector = _arg[3], revisions = _arg[4], etc = 6 <= _arg.length ? __slice.call(_arg, 5) : [];
      return _updateOpRollback(false, config, transaction, state, collectionName, selector, updateOps, rollbackProjector, revisions, etc, done);
    },
    updateOneDoc: function(config, transaction, state, _arg, done) {
      var collectionName, error, etc, newPartialDocument, oldPartialDocument, revisionName, revisionNames, x, y, _ref;
      collectionName = _arg[0], newPartialDocument = _arg[1], oldPartialDocument = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      if (etc.length !== 0) {
        return done(new Error("too many values passed to updateOneDoc"));
      }
      if (!((collectionName != null) && (newPartialDocument != null) && (oldPartialDocument != null))) {
        return done(new Error("null or missing argument to updateOneDoc command"));
      }
      if ((oldPartialDocument._id == null) || (newPartialDocument._id == null) || !oldPartialDocument._id.equals(newPartialDocument._id)) {
        return done(new Error("both old and new partial documents must have _ids, and they must match to updateOneDoc"));
      }
      if (oldPartialDocument.revision == null) {
        return done(new Error("oldPartialDocument must contain revision information to updateOneDoc"));
      }
      revisionNames = (function() {
        var _results;
        _results = [];
        for (revisionName in oldPartialDocument.revision) {
          _results.push(revisionName);
        }
        return _results;
      })();
      _ref = _processRevisions('updateOneDoc', config, collectionName, revisionNames), error = _ref[0], x = _ref[1], y = _ref[2];
      if (error != null) {
        return done(error);
      }
      return config.db.collection(collectionName, {
        strict: true
      }, function(err, collection) {
        var key, oldSelector, value, _i, _len, _ref1, _ref2;
        if (err != null) {
          return done(err, false);
        }
        oldSelector = {};
        _ref1 = _mongoFlatten(oldPartialDocument);
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          _ref2 = _ref1[_i], key = _ref2[0], value = _ref2[1];
          oldSelector[key] = value;
        }
        return collection.count(oldSelector, function(err, count) {
          if (err != null) {
            return done(err, false);
          }
          if (count !== 1) {
            transaction.execution.info.push("updateOneDoc failed to find exactly one document (found " + count + ") matching the given oldPartialDocument using " + oldSelector + ")");
            return done(null, false);
          }
          state.safeToExecuteInstruction = true;
          return _updateTransactionState(transaction, function(err) {
            var mongoOps;
            if (err != null) {
              return done(err, null);
            }
            mongoOps = {
              $set: {},
              $unset: {},
              $inc: {}
            };
            _mongoOpsFromTo(mongoOps, oldPartialDocument, newPartialDocument);
            for (revisionName in oldPartialDocument.revision) {
              mongoOps.$inc["revision." + revisionName] = 1;
            }
            return collection.update({
              _id: oldPartialDocument._id
            }, mongoOps, _updateOneOptions, function(err, updated) {
              if (err != null) {
                return done(err, null);
              }
              if (updated !== 1) {
                transaction.execution.info.push("updateOneDoc must update exactly one document, using " + (JSON.stringify(mongoOps)) + " " + updated + " were updated");
                return done(null, false);
              }
              return done(null, true);
            });
          });
        });
      });
    },
    updateOneDocRollback: function(config, transaction, state, _arg, done) {
      var collectionName, etc, newPartialDocument, oldPartialDocument;
      collectionName = _arg[0], newPartialDocument = _arg[1], oldPartialDocument = _arg[2], etc = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
      if (etc.length !== 0) {
        return done(new Error("too many values passed to updateOneDocRollback"));
      }
      if (!((collectionName != null) && (newPartialDocument != null) && (oldPartialDocument != null))) {
        return done(new Error("null or missing argument to updateOneDocRollback command"));
      }
      if ((oldPartialDocument._id == null) || (newPartialDocument._id == null) || !oldPartialDocument._id.equals(newPartialDocument._id)) {
        return done(new Error("both old and new partial documents must have _ids, and they must match to updateOneDocRollback"));
      }
      if (oldPartialDocument.revision == null) {
        return done(new Error("oldPartialDocument must contain revision information to updateOneDocRollback"));
      }
      if (!state.safeToExecuteInstruction) {
        return done(null, true);
      } else {
        return config.db.collection(collectionName, {
          strict: true
        }, function(err, collection) {
          var revisionName, selector;
          if (err != null) {
            return done(err, false);
          }
          selector = {
            _id: oldPartialDocument._id
          };
          for (revisionName in oldPartialDocument.revision) {
            selector["revision." + revisionName] = oldPartialDocument.revision[revisionName] + 1;
          }
          return collection.count(selector, function(err, count) {
            var mongoOps;
            if (err != null) {
              return done(err, false);
            }
            if (count === 0) {
              return done(null, true);
            } else {
              mongoOps = {
                $set: {},
                $unset: {}
              };
              _mongoOpsFromTo(mongoOps, newPartialDocument, oldPartialDocument);
              for (revisionName in oldPartialDocument.revision) {
                mongoOps.$set["revision." + revisionName] = oldPartialDocument.revision[revisionName];
              }
              return collection.update({
                _id: oldPartialDocument._id
              }, mongoOps, _updateOneOptions, function(err, updated) {
                if (err != null) {
                  return done(err, null);
                }
                if (updated !== 1) {
                  transaction.execution.info.push("updateOneDocRollback must update exactly one document, using " + (JSON.stringify(mongoOps)) + " " + updated + " were updated");
                  return done(null, false);
                }
                return done(null, true);
              });
            }
          });
        });
      }
    }
  };

}).call(this);
