// Generated by CoffeeScript 1.6.3
(function() {
  var ObjectID, async, chai, committed, dbconfig, log, mongodb, should, _db;

  async = require('async');

  chai = require('chai');

  mongodb = require('mongodb');

  ObjectID = require('mongodb').ObjectID;

  log = function(x) {
    return console.log(JSON.stringify(x, null, 2));
  };

  should = chai.should();

  committed = require('./../committed');

  _db = null;

  dbconfig = {
    server: "127.0.0.1",
    port: "27017",
    database: "committedtest"
  };

  before(function(done) {
    var blockingMethod, dbaddress, errorMethod, failMethod;
    committed.register('db', committed.db);
    failMethod = function(config, transaction, state, args, done) {
      return done(null, false);
    };
    committed.register('failMethod', failMethod);
    committed.register('failMethodRollback', committed.db.pass);
    errorMethod = function(config, transaction, state, args, done) {
      return done('Supposed to fail', false);
    };
    committed.register('errorMethod', errorMethod);
    committed.register('errorMethodRollback', committed.db.pass);
    blockingMethod = function(config, transaction, state, args, done) {
      return setTimeout((function() {
        return done(null, true);
      }), 1000);
    };
    committed.register('blockingMethod', blockingMethod);
    committed.register('blockingMethodRollback', committed.db.pass);
    dbaddress = 'mongodb://' + dbconfig.server + '/' + dbconfig.database;
    return mongodb.MongoClient.connect(dbaddress, function(err, db) {
      if (err != null) {
        return done(err);
      } else {
        _db = db;
        return _db.createCollection('transactions', function(err, collection) {
          return collection.remove({}, {
            w: 1
          }, done);
        });
      }
    });
  });

  describe('Committed', function() {
    describe('start', function() {
      it('should start without error', function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion'
        }, function(err) {
          return done(err);
        });
      });
      it('should not start a second time', function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion'
        }, function(err) {
          err.message.should.have.string('committed has already been started');
          return done();
        });
      });
      it('should stop without error only if started', function(done) {
        return committed.stop(function(err) {
          should.not.exist(err);
          return committed.stop(function(err) {
            err.message.should.have.string('committed is not currently started');
            return done();
          });
        });
      });
      it('should accept the predefined named functions (once)', function(done) {
        var error;
        try {
          return committed.register('db', committed.db);
        } catch (_error) {
          error = _error;
          return error.message.should.have.string('path db already exists in registry');
        } finally {
          done();
        }
      });
      return it('should not be possible to duplicate a registered function name', function(done) {
        var error;
        try {
          return committed.register('db.insert', 'foo');
        } catch (_error) {
          error = _error;
          return error.message.should.have.string('path db.insert already exists in registry');
        } finally {
          done();
        }
      });
    });
    describe('valid operation', function() {
      before(function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            validOpsTest: ['number']
          }
        }, function(err) {
          should.not.exist(err);
          return _db.createCollection('validOpsTest', done);
        });
      });
      after(function(done) {
        return committed.stop(done);
      });
      it('should be able to sequentially process multiple operations in a single queue', function(done) {
        var start, task;
        task = function(n, next) {
          var transaction;
          transaction = committed.transaction();
          transaction.queue = 'testQ';
          transaction.instructions.push({
            name: 'db.insert',
            "arguments": [
              'validOpsTest', {
                value: n + 10
              }
            ]
          });
          return committed.enqueue(transaction, next);
        };
        start = new Date().getTime();
        return async.times(20, task, function(err) {
          var transactionTime;
          transactionTime = new Date().getTime() - start;
          should.not.exist(err);
          task = function(n, next) {
            return _db.collection('validOpsTest').insert({
              value: n + 20
            }, {
              w: 1,
              journal: true,
              upsert: false,
              multi: false,
              serializeFunctions: false
            }, next);
          };
          start = new Date().getTime();
          return async.timesSeries(20, task, function(err) {
            var directTime;
            should.not.exist(err);
            directTime = new Date().getTime() - start;
            (transactionTime / directTime).should.be.below(5);
            return done();
          });
        });
      });
      return it('should be possible to place transactions in multiple queues', function(done) {
        var t, tasks, _fn, _i;
        tasks = [];
        _fn = function(t) {
          var q, _j, _results;
          _results = [];
          for (q = _j = 1; _j <= 10; q = ++_j) {
            _results.push((function(q) {
              return tasks.push(function(done) {
                var transaction;
                transaction = committed.transaction("testQ" + q, {}, [
                  {
                    name: 'db.insert',
                    "arguments": [
                      'validOpsTest', {
                        t: t,
                        q: q
                      }
                    ]
                  }
                ]);
                return committed.enqueue(transaction, function(err, status) {
                  should.not.exist(err);
                  status.should.be.string('Committed');
                  return done();
                });
              });
            })(q));
          }
          return _results;
        };
        for (t = _i = 1; _i <= 10; t = ++_i) {
          _fn(t);
        }
        return async.parallel(tasks, done);
      });
    });
    describe('global lock', function() {
      before(function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            globalLockTest: ['number']
          }
        }, function(err) {
          should.not.exist(err);
          return _db.createCollection('globalLockTest', done);
        });
      });
      after(function(done) {
        return committed.stop(done);
      });
      it('should let normal queues process after a GlobalLock transaction', function(done) {
        var globalTransaction;
        globalTransaction = committed.transaction('GlobalLock', {});
        globalTransaction.instructions.push({
          name: 'blockingMethod'
        });
        return committed.withGlobalLock(globalTransaction, function(err, status) {
          var nonGlobalTransaction;
          should.not.exist(err);
          status.should.equal('Committed');
          nonGlobalTransaction = committed.transaction('nonGlobal', {}, [], []);
          return committed.enqueue(nonGlobalTransaction, function(err, status) {
            status.should.equal('Committed');
            return done(err);
          });
        });
      });
      it('should let immediate transactions process after a GlobalLock transaction', function(done) {
        var globalTransaction;
        globalTransaction = committed.transaction('GlobalLock', {}, [
          {
            name: 'blockingMethod'
          }
        ]);
        return committed.withGlobalLock(globalTransaction, function(err, status) {
          var immediateTransaction;
          should.not.exist(err);
          status.should.equal('Committed');
          immediateTransaction = committed.transaction();
          return committed.immediately(immediateTransaction, function(err, status) {
            status.should.equal('Committed');
            return done(err);
          });
        });
      });
      it('should wait for transactions in other queues to finish before starting a global lock transaction', function(done) {
        var d, doc1, doc2, doc3, globalLockTestMethod, gt, ot1, ot2, ot3;
        d = new Date();
        doc1 = {
          now: d
        };
        doc2 = {
          now: d
        };
        doc3 = {
          now: d
        };
        ot1 = committed.transaction('q1', {}, [
          {
            name: 'blockingMethod'
          }, {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc1]
          }
        ]);
        ot2 = committed.transaction('q2', {}, [
          {
            name: 'blockingMethod'
          }, {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc2]
          }
        ]);
        ot3 = committed.transaction('q3', {}, [
          {
            name: 'blockingMethod'
          }, {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc3]
          }
        ]);
        globalLockTestMethod = function(config, transaction, state, args, instructionDone) {
          return config.db.collection('globalLockTest', {
            strict: true
          }, function(err, collection) {
            if (err != null) {
              return instructionDone(err);
            }
            return collection.find({
              now: d
            }).toArray(function(err, docs) {
              docs.length.should.equal(3);
              return instructionDone(null, true);
            });
          });
        };
        committed.register('globalLockTestMethod', globalLockTestMethod);
        committed.register('globalLockTestMethodRollback', committed.db.pass);
        gt = committed.transaction('GlobalLock', {}, [
          {
            name: 'globalLockTestMethod'
          }
        ]);
        committed.enqueue(ot1, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.enqueue(ot2, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.enqueue(ot3, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        return committed.withGlobalLock(gt, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return done();
        });
      });
      it('should wait for immediate transactions to complete before starting a global lock', function(done) {
        var doc, globalImmediateTestMethod, gt, im;
        doc = {
          now: new Date()
        };
        im = committed.transaction(null, {}, [
          {
            name: 'blockingMethod'
          }, {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc]
          }
        ]);
        globalImmediateTestMethod = function(config, transaction, state, args, instructionDone) {
          return config.db.collection('globalLockTest', {
            strict: true
          }, function(err, collection) {
            if (err != null) {
              return instructionDone(err);
            }
            return collection.find(doc).toArray(function(err, docs) {
              docs.length.should.equal(1);
              docs[0].now.should.deep.equal(doc.now);
              return instructionDone(null, true);
            });
          });
        };
        committed.register('globalImmediateTestMethod', globalImmediateTestMethod);
        committed.register('globalImmediateTestMethodRollback', committed.db.pass);
        gt = committed.transaction('GlobalLock', {}, [
          {
            name: 'globalImmediateTestMethod'
          }
        ]);
        committed.immediately(im, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        return committed.withGlobalLock(gt, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return done();
        });
      });
      it('should pend ordinary queues during globallock and resume them after', function(done) {
        var d, doc1, doc2, doc3, gt1, gt2, gt3, ot, pendingTestMethod;
        d = new Date();
        doc1 = {
          now: d
        };
        doc2 = {
          now: d
        };
        doc3 = {
          now: d
        };
        gt1 = committed.transaction('GlobalLock', {}, [
          {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc1]
          }
        ]);
        gt2 = committed.transaction('GlobalLock', {}, [
          {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc2]
          }
        ]);
        gt3 = committed.transaction('GlobalLock', {}, [
          {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc3]
          }
        ]);
        pendingTestMethod = function(config, transaction, state, args, instructionDone) {
          return config.db.collection('globalLockTest', {
            strict: true
          }, function(err, collection) {
            if (err != null) {
              return instructionDone(err);
            }
            return collection.find({
              now: d
            }).toArray(function(err, docs) {
              docs.length.should.equal(3);
              return instructionDone(null, true);
            });
          });
        };
        committed.register('pendingTestMethod', pendingTestMethod);
        committed.register('pendingTestMethodRollback', committed.db.pass);
        ot = committed.transaction('Q', {}, [
          {
            name: 'pendingTestMethod'
          }
        ]);
        committed.withGlobalLock(gt1, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.withGlobalLock(gt2, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.withGlobalLock(gt3, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        return committed.enqueue(ot, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return done();
        });
      });
      return it('should pend immediately transactions during globallock and resume after', function(done) {
        var d, doc1, doc2, doc3, gt1, gt2, gt3, im, pendingImmediateTestMethod;
        d = new Date();
        doc1 = {
          now: d
        };
        doc2 = {
          now: d
        };
        doc3 = {
          now: d
        };
        gt1 = committed.transaction('GlobalLock', {}, [
          {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc1]
          }
        ]);
        gt2 = committed.transaction('GlobalLock', {}, [
          {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc2]
          }
        ]);
        gt3 = committed.transaction('GlobalLock', {}, [
          {
            name: 'db.insert',
            "arguments": ['globalLockTest', doc3]
          }
        ]);
        pendingImmediateTestMethod = function(config, transaction, state, args, instructionDone) {
          return config.db.collection('globalLockTest', {
            strict: true
          }, function(err, collection) {
            if (err != null) {
              return instructionDone(err);
            }
            return collection.find({
              now: d
            }).toArray(function(err, docs) {
              docs.length.should.equal(3);
              return instructionDone(null, true);
            });
          });
        };
        committed.register('pendingImmediateTestMethod', pendingImmediateTestMethod);
        committed.register('pendingImmediateTestMethodRollback', committed.db.pass);
        im = committed.transaction(null, {}, [
          {
            name: 'pendingTestMethod'
          }
        ]);
        committed.withGlobalLock(gt1, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.withGlobalLock(gt2, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.withGlobalLock(gt3, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        return committed.immediately(im, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return done();
        });
      });
    });
    describe('stop', function() {
      beforeEach(function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            stopTest: ['number']
          }
        }, function(err) {
          should.not.exist(err);
          return _db.createCollection('stopTest', function(err, collection) {
            if (err != null) {
              return done(err);
            }
            return collection.remove({}, {
              w: 1
            }, done);
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      return it('should wait for immediate, enqueued, and global lock tasks to complete before stopping', function(done) {
        var doc1, doc2, doc3, gt, im, ot;
        doc1 = {
          inserted: 'beforeStop'
        };
        doc2 = {
          inserted: 'beforeStop'
        };
        doc3 = {
          inserted: 'beforeStop'
        };
        im = committed.transaction(null, {}, [
          {
            name: 'db.insert',
            "arguments": ['stopTest', doc1]
          }
        ]);
        gt = committed.transaction('GlobalLock', {}, [
          {
            name: 'db.insert',
            "arguments": ['stopTest', doc2]
          }
        ]);
        ot = committed.transaction('OrdinaryQueue', {}, [
          {
            name: 'db.insert',
            "arguments": ['stopTest', doc3]
          }
        ]);
        committed.immediately(im, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.enqueue(ot, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.withGlobalLock(gt, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        return committed.stop(function(err) {
          should.not.exist(err);
          return _db.collection('stopTest', {
            strict: true
          }, function(err, collection) {
            return collection.find({
              inserted: 'beforeStop'
            }).toArray(function(err, docs) {
              docs.length.should.equal(3);
              return done();
            });
          });
        });
      });
    });
    describe('rollback', function() {
      before(function(done) {
        var failSlowlyMethod, failSlowlyMethodRollback;
        failSlowlyMethod = function(config, transaction, state, args, done) {
          return setTimeout((function() {
            return done(null, false);
          }), 1000);
        };
        committed.register('failSlowlyMethod', failSlowlyMethod);
        failSlowlyMethodRollback = function(config, transaction, state, args, done) {
          return setTimeout((function() {
            return done(null, true);
          }), 1000);
        };
        committed.register('failSlowlyMethodRollback', failSlowlyMethodRollback);
        return done();
      });
      beforeEach(function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            rollbackTest: ['number']
          }
        }, function(err) {
          should.not.exist(err);
          return _db.createCollection('rollbackTest', function(err, collection) {
            return collection.remove({}, {
              w: 1
            }, done);
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      it('should rollback an errored transaction and return status FailedCommitErrorRollbackOk', function(done) {
        var docA, transaction;
        docA = {
          foo: true
        };
        transaction = committed.transaction('rollbackTest', {}, [
          {
            name: 'db.insert',
            "arguments": ['rollbackTest', docA]
          }, {
            name: 'errorMethod'
          }
        ], [
          {
            name: 'db.insertRollback',
            "arguments": ['rollbackTest', docA]
          }, {
            name: 'db.pass'
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          should.not.exist(err);
          status.should.equal('FailedCommitErrorRollbackOk');
          return _db.collection('rollbackTest').count(docA, function(err, count) {
            count.should.equal(0);
            return done(err);
          });
        });
      });
      it('should not execute rollbacks for instructions which were not executed, nor execute further instructions after an error', function(done) {
        var docA, docB, transaction;
        docA = {
          foo: true
        };
        docB = {
          bar: true
        };
        transaction = committed.transaction('rollbackTest', {}, [
          {
            name: 'db.insert',
            "arguments": ['rollbackTest', docA]
          }, {
            name: 'errorMethod'
          }, {
            name: 'db.insert',
            "arguments": ['rollbackTest', docB]
          }
        ], [
          {
            name: 'db.insertRollback',
            "arguments": ['rollbackTest', docA]
          }, {
            name: 'db.pass'
          }, {
            name: 'errorMethod'
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          should.not.exist(err);
          status.should.equal('FailedCommitErrorRollbackOk');
          return _db.collection('rollbackTest').count({
            _id: {
              $in: [docA._id, docB._id]
            }
          }, function(err, count) {
            count.should.equal(0);
            return done(err);
          });
        });
      });
      it('should rollback a failed transaction and return status Failed', function(done) {
        var doc, transaction;
        doc = {
          rolledback: false
        };
        transaction = committed.transaction('rollbackTest', {}, [
          {
            name: 'db.insert',
            "arguments": ['rollbackTest', doc]
          }, {
            name: 'failMethod'
          }
        ], [
          {
            name: 'db.updateOneOp',
            "arguments": [
              'rollbackTest', doc, {
                $set: {
                  rolledback: true
                }
              }, {}
            ]
          }, {
            name: 'db.pass'
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          status.should.equal('Failed');
          return _db.collection('rollbackTest').count({
            _id: doc._id,
            rolledback: true
          }, function(err, count) {
            count.should.equal(1);
            return done(err);
          });
        });
      });
      it('should report a catastrophe if a rollback fails after an error', function(done) {
        var doc, transaction;
        doc = {
          content: 'agogo'
        };
        transaction = committed.transaction('rollbackTest', {}, [
          {
            name: 'db.insert',
            "arguments": ['rollbackTest', doc]
          }, {
            name: 'errorMethod'
          }
        ], [
          {
            name: 'errorMethod'
          }, {
            name: 'errorMethod'
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          status.should.equal('CatastropheCommitErrorRollbackError');
          err.should.equal('Supposed to fail');
          return done();
        });
      });
      it('should report a catastrophe if a rollback fails after a failure', function(done) {
        var doc, transaction;
        doc = {
          content: 'agogo'
        };
        transaction = committed.transaction('rollbackTest', {}, [
          {
            name: 'db.insert',
            "arguments": ['rollbackTest', doc]
          }, {
            name: 'failMethod'
          }
        ], [
          {
            name: 'failMethod'
          }, {
            name: 'failMethod'
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          status.should.equal('CatastropheCommitFailedRollbackError');
          return done(err);
        });
      });
      return it('should use implicit rollbacks when the transaction.rollback array is null', function(done) {
        var transaction;
        transaction = committed.transaction('rollbackTest', {}, [
          {
            name: 'db.insert',
            "arguments": [
              'rollbackTest', {
                content: 'great content'
              }
            ]
          }, {
            name: 'db.insert',
            "arguments": [
              'rollbackTest', {
                content: 'more great content'
              }
            ]
          }, {
            name: 'failSlowlyMethod',
            "arguments": []
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          should.not.exist(err);
          status.should.equal('Failed');
          return _db.collection('rollbackTest').find().toArray(function(err, docs) {
            docs.length.should.equal(0);
            return done();
          });
        });
      });
    });
    describe('error handling', function() {
      before(function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion'
        }, done);
      });
      after(function(done) {
        return committed.stop(done);
      });
      it('should reject a transaction for processing with an invalid queue name', function(done) {
        var tests;
        tests = [];
        tests.push(function(done) {
          var globalTransaction;
          globalTransaction = committed.transaction('GlobalLock');
          return committed.enqueue(globalTransaction, function(err) {
            err.message.should.have.string("Can't queue a transaction for GlobalLock using the committed.enqueue function");
            return done();
          });
        });
        tests.push(function(done) {
          var nullQueueTransaction;
          nullQueueTransaction = committed.transaction(null);
          return committed.enqueue(nullQueueTransaction, function(err) {
            err.message.should.have.string('must have a transaction.queue parameter to use committed.enqueue');
            return done();
          });
        });
        tests.push(function(done) {
          var emptyQueueTransaction;
          emptyQueueTransaction = committed.transaction('');
          return committed.enqueue(emptyQueueTransaction, function(err) {
            err.message.should.have.string('must have a transaction.queue parameter to use committed.enqueue');
            return done();
          });
        });
        tests.push(function(done) {
          var nonGlobalTransaction;
          nonGlobalTransaction = committed.transaction('testQ');
          return committed.withGlobalLock(nonGlobalTransaction, function(err) {
            err.message.should.have.string("Can't call committed.withGlobalLock for a transaction that names a queue other than 'GlobalLock'");
            return done();
          });
        });
        tests.push(function(done) {
          var immediateTransaction;
          immediateTransaction = committed.transaction('testQ');
          return committed.immediately(immediateTransaction, function(err) {
            err.message.should.have.string("Can't call committed.immediately on a transaction which has a queue defined for it");
            return done();
          });
        });
        return async.parallel(tests, done);
      });
      return it('should return an err if transactions are enqueued when not started', function(done) {
        return committed.stop(function(err) {
          var tests;
          should.not.exist(err);
          tests = [];
          tests.push(function(done) {
            var transaction;
            transaction = committed.transaction('testQ');
            return committed.enqueue(transaction, function(err, status) {
              should.exist(err);
              return done();
            });
          });
          tests.push(function(done) {
            var transaction;
            transaction = committed.transaction();
            return committed.immediately(transaction, function(err, status) {
              should.exist(err);
              return done();
            });
          });
          tests.push(function(done) {
            var transaction;
            transaction = committed.transaction('GlobalLock');
            return committed.withGlobalLock(transaction, function(err, status) {
              should.exist(err);
              return done();
            });
          });
          return async.parallel(tests, function(err) {
            if (err != null) {
              return done(err);
            }
            return committed.start({
              db: _db,
              softwareVersion: 'testSoftwareVersion'
            }, done);
          });
        });
      });
    });
    describe('instructions', function() {
      beforeEach(function(done) {
        var config;
        config = {
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            instructionsTest: ['content', 'otherContent']
          }
        };
        return committed.start(config, function(err) {
          should.not.exist(err);
          return _db.createCollection('instructionsTest', function(err, collection) {
            return collection.remove({}, {
              w: 1
            }, done);
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      it('insert should reach mongo, and be issued with an _id', function(done) {
        var doc, transaction;
        doc = {
          t: 't',
          q: 'q'
        };
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', doc]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          should.exist(doc._id);
          return _db.collection('instructionsTest', function(err, collection) {
            should.not.exist(err);
            return collection.findOne({
              _id: doc._id
            }, function(err, docInDB) {
              should.not.exist(err);
              doc.t.should.equal(docInDB.t);
              doc.q.should.equal(docInDB.q);
              return done();
            });
          });
        });
      });
      it('insert should work with multiple documents', function(done) {
        var docs, i, transaction;
        docs = (function() {
          var _i, _results;
          _results = [];
          for (i = _i = 0; _i <= 10; i = ++_i) {
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', docs]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          return _db.collection('instructionsTest', function(err, collection) {
            var doc, ids;
            should.not.exist(err);
            ids = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = docs.length; _i < _len; _i++) {
                doc = docs[_i];
                _results.push(doc._id);
              }
              return _results;
            })();
            return collection.find({
              _id: {
                $in: ids
              }
            }).toArray(function(err, docsInDB) {
              should.not.exist(err);
              docsInDB.length.should.equal(docs.length);
              return done();
            });
          });
        });
      });
      it('insertRollback should undo insert for multiple documents', function(done) {
        var docs, i, transaction;
        docs = (function() {
          var _i, _results;
          _results = [];
          for (i = _i = 0; _i <= 10; i = ++_i) {
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', docs]
          }, {
            name: 'failMethod'
          }
        ], [
          {
            name: 'db.insertRollback',
            "arguments": ['instructionsTest', docs]
          }, {
            name: 'db.pass'
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Failed');
          return _db.collection('instructionsTest', function(err, collection) {
            var doc, ids;
            should.not.exist(err);
            ids = (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = docs.length; _i < _len; _i++) {
                doc = docs[_i];
                _results.push(doc._id);
              }
              return _results;
            })();
            return collection.find({
              _id: {
                $in: ids
              }
            }).toArray(function(err, docsInDB) {
              should.not.exist(err);
              docsInDB.length.should.equal(0);
              return done();
            });
          });
        });
      });
      it('updateOneOp should update one document, without specified revisions all are incremented', function(done) {
        var docs, i, transaction;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', docs]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          var d, ids, update;
          should.not.exist(err);
          status.should.be.string('Committed');
          ids = (function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = docs.length; _i < _len; _i++) {
              d = docs[_i];
              _results.push(d._id);
            }
            return _results;
          })();
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateOneOp',
              "arguments": [
                'instructionsTest', {
                  _id: {
                    $in: ids
                  },
                  i: {
                    $gt: 2
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Committed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                j: 1
              }).toArray(function(err, docsInDB) {
                should.not.exist(err);
                docsInDB.length.should.equal(1);
                docsInDB[0].revision.content.should.equal(2);
                docsInDB[0].revision.otherContent.should.equal(2);
                return done();
              });
            });
          });
        });
      });
      it('updateOneOpRollback should undo an updateOneOp', function(done) {
        var docs, i, transaction;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', docs]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          var d, ids, update;
          should.not.exist(err);
          status.should.be.string('Committed');
          ids = (function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = docs.length; _i < _len; _i++) {
              d = docs[_i];
              _results.push(d._id);
            }
            return _results;
          })();
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateOneOp',
              "arguments": [
                'instructionsTest', {
                  _id: {
                    $in: ids
                  },
                  i: {
                    $gt: 2
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }, {
              name: 'failMethod'
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Failed');
            return _db.collection('instructionsTest', function(err, collection) {
              should.not.exist(err);
              return collection.find({
                _id: {
                  $in: ids
                },
                i: {
                  $gt: 2
                }
              }).toArray(function(err, docsInDB) {
                should.not.exist(err);
                docsInDB.length.should.equal(1);
                docsInDB[0].revision.content.should.equal(1);
                docsInDB[0].revision.otherContent.should.equal(1);
                should.not.exist(docsInDB[0].j);
                return done();
              });
            });
          });
        });
      });
      it('updateOneOp should not update a document thats inconsistent with a given revision', function(done) {
        var doc, transaction;
        doc = {
          i: 1
        };
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', doc]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          var update;
          should.not.exist(err);
          status.should.be.string('Committed');
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateOneOp',
              "arguments": [
                'instructionsTest', {
                  _id: doc._id
                }, {
                  $inc: {
                    j: 1
                  }
                }, {}, 'content'
              ]
            }, {
              name: 'db.updateOneOp',
              "arguments": [
                'instructionsTest', {
                  _id: doc._id
                }, {
                  $inc: {
                    j: 1
                  }
                }, {}, {
                  'content': 0
                }
              ]
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Failed');
            return _db.collection('instructionsTest', function(err, collection) {
              should.not.exist(err);
              return collection.findOne({
                _id: doc._id
              }, function(err, result) {
                should.not.exist(err);
                result.revision.content.should.equal(1);
                result.revision.otherContent.should.equal(1);
                result.i.should.equal(1);
                should.not.exist(result.j);
                return done();
              });
            });
          });
        });
      });
      it('updateOneOp should only increment specified revisions', function(done) {
        var doc, transaction;
        doc = {
          i: 1
        };
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', doc]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          var update;
          should.not.exist(err);
          status.should.be.string('Committed');
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateOneOp',
              "arguments": [
                'instructionsTest', {
                  _id: doc._id
                }, {
                  $set: {
                    j: 1
                  }
                }, {}, 'content'
              ]
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Committed');
            return _db.collection('instructionsTest', function(err, collection) {
              should.not.exist(err);
              return collection.findOne({
                _id: doc._id
              }, function(err, result) {
                should.not.exist(err);
                result.revision.content.should.equal(2);
                result.revision.otherContent.should.equal(1);
                result.i.should.equal(1);
                result.j.should.equal(1);
                return done();
              });
            });
          });
        });
      });
      it('updateOneOp should fail, when no documents match the selector', function(done) {
        var update;
        update = committed.transaction("test", {}, [
          {
            name: 'db.updateOneOp',
            "arguments": [
              'instructionsTest', {
                mikey: {
                  $exists: true
                }
              }, {
                mikey: 'should not be here'
              }
            ]
          }
        ]);
        return committed.enqueue(update, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Failed');
          return _db.collection('instructionsTest', function(err, collection) {
            return collection.count({
              mikey: {
                $exists: true
              }
            }, function(err, count) {
              should.not.exist(err);
              count.should.equal(0);
              return done();
            });
          });
        });
      });
      it('updateOneOp should use a projection for rollback when specified');
      it('updateOneOp should return errors when given a bad updateOps object');
      it('updateOneOp should deal with complex rollback projections which set and unset keys');
      it('updateOneOp should be reasonably performant');
      it('updateOneDoc should update a document with a correct revision', function(done) {
        var insert, insertDoc, newDoc, oldDoc;
        insertDoc = {
          content: 'here',
          otherContent: "other"
        };
        oldDoc = {
          revision: {
            content: 1
          },
          content: 'here'
        };
        newDoc = {
          revision: {
            content: 100000
          },
          content: 'here again'
        };
        insert = committed.transaction("test");
        insert.instructions.push({
          name: 'db.insert',
          "arguments": ['instructionsTest', insertDoc]
        });
        return committed.enqueue(insert, function(err, status) {
          var update;
          should.not.exist(err);
          status.should.be.string('Committed');
          oldDoc._id = newDoc._id = insertDoc._id;
          update = committed.transaction("test", {});
          update.instructions.push({
            name: 'db.updateOneDoc',
            "arguments": ['instructionsTest', newDoc, oldDoc]
          });
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Committed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                content: "here again"
              }).toArray(function(err, docsInDB) {
                docsInDB.length.should.equal(1);
                should.exist(docsInDB[0]._id);
                delete docsInDB[0]._id;
                docsInDB[0].should.deep.equal({
                  revision: {
                    content: 2,
                    otherContent: 1
                  },
                  content: 'here again',
                  otherContent: "other"
                });
                return done();
              });
            });
          });
        });
      });
      it('updateOneDocRollback should undo a updateOneDoc', function(done) {
        var insert, insertDoc, newDoc, oldDoc;
        insertDoc = {
          content: 'here',
          otherContent: "other"
        };
        oldDoc = {
          revision: {
            content: 1
          },
          content: 'here'
        };
        newDoc = {
          revision: {
            content: 1
          },
          content: 'here again'
        };
        insert = committed.transaction("test");
        insert.instructions.push({
          name: 'db.insert',
          "arguments": ['instructionsTest', insertDoc]
        });
        return committed.enqueue(insert, function(err, status) {
          var update;
          should.not.exist(err);
          status.should.be.string('Committed');
          oldDoc._id = newDoc._id = insertDoc._id;
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateOneDoc',
              "arguments": ['instructionsTest', newDoc, oldDoc]
            }, {
              name: 'failMethod'
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Failed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                content: "here"
              }).toArray(function(err, docsInDB) {
                docsInDB.length.should.equal(1);
                should.exist(docsInDB[0]._id);
                delete docsInDB[0]._id;
                docsInDB[0].should.deep.equal({
                  revision: {
                    content: 1,
                    otherContent: 1
                  },
                  content: 'here',
                  otherContent: "other"
                });
                return done();
              });
            });
          });
        });
      });
      it('updateOneDoc should not update a document whose revision has changed', function(done) {
        var insert, insertDoc, newDoc, oldDoc;
        insertDoc = {
          content: 'here',
          otherContent: "other"
        };
        oldDoc = {
          revision: {
            content: 1
          },
          content: 'here'
        };
        newDoc = {
          revision: {
            content: 1
          },
          content: 'here again'
        };
        insert = committed.transaction("test");
        insert.instructions.push({
          name: 'db.insert',
          "arguments": ['instructionsTest', insertDoc]
        });
        return committed.enqueue(insert, function(err, status) {
          var update;
          should.not.exist(err);
          status.should.be.string('Committed');
          oldDoc._id = newDoc._id = insertDoc._id;
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateOneDoc',
              "arguments": ['instructionsTest', newDoc, oldDoc]
            }, {
              name: 'db.updateOneDoc',
              "arguments": ['instructionsTest', newDoc, oldDoc]
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Failed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                content: "here"
              }).toArray(function(err, docsInDB) {
                docsInDB.length.should.equal(1);
                should.exist(docsInDB[0]._id);
                delete docsInDB[0]._id;
                docsInDB[0].should.deep.equal({
                  revision: {
                    content: 1,
                    otherContent: 1
                  },
                  content: 'here',
                  otherContent: "other"
                });
                return done();
              });
            });
          });
        });
      });
      it('updateManyOp should update many documents', function(done) {
        var docs, i, insert;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        insert = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', docs]
          }
        ]);
        return committed.enqueue(insert, function(err, status) {
          var d, ids, update;
          should.not.exist(err);
          status.should.be.string('Committed');
          ids = (function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = docs.length; _i < _len; _i++) {
              d = docs[_i];
              _results.push(d._id);
            }
            return _results;
          })();
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateManyOp',
              "arguments": [
                'instructionsTest', {
                  _id: {
                    $in: ids
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Committed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                j: 1
              }).toArray(function(err, docsInDB) {
                var _i, _len;
                should.not.exist(err);
                docsInDB.length.should.equal(3);
                for (_i = 0, _len = docsInDB.length; _i < _len; _i++) {
                  d = docsInDB[_i];
                  d.revision.content.should.equal(2);
                  d.revision.otherContent.should.equal(2);
                }
                return done();
              });
            });
          });
        });
      });
      it('updateManyOpRollback should rollback many updates', function(done) {
        var docs, i, insert;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        insert = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', docs]
          }
        ]);
        return committed.enqueue(insert, function(err, status) {
          var d, ids, update;
          should.not.exist(err);
          status.should.be.string('Committed');
          ids = (function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = docs.length; _i < _len; _i++) {
              d = docs[_i];
              _results.push(d._id);
            }
            return _results;
          })();
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateManyOp',
              "arguments": [
                'instructionsTest', {
                  _id: {
                    $in: ids
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }, {
              name: 'failMethod'
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Failed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                _id: {
                  $in: ids
                }
              }).toArray(function(err, docsInDB) {
                var _i, _len;
                should.not.exist(err);
                docsInDB.length.should.equal(3);
                for (_i = 0, _len = docsInDB.length; _i < _len; _i++) {
                  d = docsInDB[_i];
                  d.revision.content.should.equal(1);
                  d.revision.otherContent.should.equal(1);
                  should.not.exist(d.j);
                }
                return done();
              });
            });
          });
        });
      });
      it('updateManyOp should update no documents, when no documents match the selector, without failing', function(done) {
        var update;
        update = committed.transaction("test", {}, [
          {
            name: 'db.updateManyOp',
            "arguments": [
              'instructionsTest', {
                mikey: {
                  $exists: true
                }
              }, {
                mikey: 'should not be here'
              }
            ]
          }
        ]);
        return committed.enqueue(update, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          return _db.collection('instructionsTest', function(err, collection) {
            return collection.count({
              mikey: {
                $exists: true
              }
            }, function(err, count) {
              should.not.exist(err);
              count.should.equal(0);
              return done();
            });
          });
        });
      });
      it('upsertOneOp should update a document', function(done) {
        var doc, transaction;
        doc = {
          my_date: new Date()
        };
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', doc]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          var update;
          should.not.exist(err);
          status.should.be.string('Committed');
          update = committed.transaction("test", {}, [
            {
              name: 'db.upsertOneOp',
              "arguments": [
                'instructionsTest', {
                  my_date: doc.my_date
                }, {
                  $set: {
                    updated: true
                  }
                }
              ]
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Committed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                _id: doc._id
              }).toArray(function(err, docsInDB) {
                should.not.exist(err);
                docsInDB.length.should.equal(1);
                docsInDB[0].revision.content.should.equal(2);
                docsInDB[0].revision.otherContent.should.equal(2);
                docsInDB[0].updated.should.be["true"];
                return done();
              });
            });
          });
        });
      });
      it('upsertOneOp should insert a document', function(done) {
        var upsert, _id;
        _id = new ObjectID();
        upsert = committed.transaction("test", {}, [
          {
            name: 'db.upsertOneOp',
            "arguments": [
              'instructionsTest', {
                _id: _id
              }, {
                $set: {
                  inserted: true
                }
              }
            ]
          }
        ]);
        return committed.enqueue(upsert, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          return _db.collection('instructionsTest', function(err, collection) {
            return collection.find({
              _id: _id
            }).toArray(function(err, docsInDB) {
              should.not.exist(err);
              docsInDB.length.should.equal(1);
              docsInDB[0].revision.content.should.equal(1);
              docsInDB[0].revision.otherContent.should.equal(1);
              docsInDB[0].inserted.should.be["true"];
              return done();
            });
          });
        });
      });
      it('upsertOneOp should rollback after an update', function(done) {
        var doc, transaction;
        doc = {
          my_date: new Date()
        };
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['instructionsTest', doc]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          var update;
          should.not.exist(err);
          status.should.be.string('Committed');
          update = committed.transaction("test", {}, [
            {
              name: 'db.upsertOneOp',
              "arguments": [
                'instructionsTest', {
                  my_date: doc.my_date
                }, {
                  $set: {
                    updated: true
                  }
                }
              ]
            }, {
              name: 'failMethod'
            }
          ]);
          return committed.enqueue(update, function(err, status) {
            should.not.exist(err);
            status.should.be.string('Failed');
            return _db.collection('instructionsTest', function(err, collection) {
              return collection.find({
                _id: doc._id
              }).toArray(function(err, docsInDB) {
                should.not.exist(err);
                docsInDB.length.should.equal(1);
                docsInDB[0].revision.content.should.equal(1);
                docsInDB[0].revision.otherContent.should.equal(1);
                should.not.exist(docsInDB[0].updated);
                return done();
              });
            });
          });
        });
      });
      return it('upsertOneOp should rollback after an insert', function(done) {
        var upsert, _id;
        _id = new ObjectID();
        upsert = committed.transaction("test", {}, [
          {
            name: 'db.upsertOneOp',
            "arguments": [
              'instructionsTest', {
                _id: _id
              }, {
                $set: {
                  inserted: true
                }
              }
            ]
          }, {
            name: 'failMethod'
          }
        ]);
        return committed.enqueue(upsert, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Failed');
          return _db.collection('instructionsTest', function(err, collection) {
            return collection.find({
              _id: _id
            }).toArray(function(err, docsInDB) {
              should.not.exist(err);
              docsInDB.length.should.equal(0);
              return done();
            });
          });
        });
      });
    });
    describe('chained transactions', function() {
      beforeEach(function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            chainedTest: ['number']
          }
        }, function(err) {
          should.not.exist(err);
          return _db.createCollection('chainedTest', function(err, collection) {
            return collection.remove({}, {
              w: 1
            }, done);
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      it('should execute all the instructions in each transaction in the chain', function(done) {
        var bigT;
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q3', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ]);
        return committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return _db.collection('chainedTest', {
            strict: true
          }, function(err, collection) {
            should.not.exist(err);
            return collection.find({
              execute: 'all'
            }).toArray(function(err, docs) {
              docs.length.should.equal(6);
              return done();
            });
          });
        });
      });
      it('should execute their transactions in sequence, and within the sequence of other queues', function(done) {
        var bigT, chainQueueCheck, q1, q2, q3;
        chainQueueCheck = function(config, transaction, state, _arg, done) {
          var what;
          what = _arg[0];
          return _db.collection('chainedTest', function(err, collection) {
            should.exist(what);
            return collection.find({
              where: what
            }).toArray(function(err, docs) {
              docs.length.should.equal(1);
              return done(null, true);
            });
          });
        };
        committed.register('chainQueueCheck', chainQueueCheck);
        committed.register('chainQueueCheckRollback', committed.db.pass);
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'chainQueueCheck',
              "arguments": ['q1']
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'chainQueueCheck',
              "arguments": ['q2']
            }
          ]), committed.transaction('q3', {}, [
            {
              name: 'chainQueueCheck',
              "arguments": ['q3']
            }
          ])
        ]);
        q1 = committed.transaction('q1', {}, [
          {
            name: "blockingMethod"
          }, {
            name: 'db.insert',
            "arguments": [
              'chainedTest', {
                where: 'q1'
              }
            ]
          }
        ]);
        q2 = committed.transaction('q2', {}, [
          {
            name: "blockingMethod"
          }, {
            name: 'db.insert',
            "arguments": [
              'chainedTest', {
                where: 'q2'
              }
            ]
          }
        ]);
        q3 = committed.transaction('q3', {}, [
          {
            name: "blockingMethod"
          }, {
            name: 'db.insert',
            "arguments": [
              'chainedTest', {
                where: 'q3'
              }
            ]
          }
        ]);
        committed.enqueue(q1, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.enqueue(q2, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        committed.enqueue(q3, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        return committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return done();
        });
      });
      it('should return Failed if the first transaction in the chain fails, and roll it back', function(done) {
        var bigT;
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'failMethod'
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q3', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ]);
        return committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Failed');
          return _db.collection('chainedTest', function(err, collection) {
            return collection.find({
              execute: 'all'
            }).toArray(function(err, docs) {
              docs.length.should.equal(0);
              return done();
            });
          });
        });
      });
      it('should return ChainFailed if a non-first transaction fails, and rollback only that failed transaction', function(done) {
        var bigT;
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'failMethod'
            }
          ]), committed.transaction('q3', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ]);
        return committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('ChainFailed');
          return _db.collection('chainedTest', function(err, collection) {
            return collection.find({
              execute: 'all'
            }).toArray(function(err, docs) {
              docs.length.should.equal(2);
              return done();
            });
          });
        });
      });
      it('should finish executing a chain before allowing exports.stop to callback', function(done) {
        var bigT;
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q3', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ]);
        committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          return status.should.equal('Committed');
        });
        return committed.stop(function(err) {
          should.not.exist(err);
          return _db.collection('chainedTest', function(err, collection) {
            return collection.find({
              execute: 'all'
            }).toArray(function(err, docs) {
              docs.length.should.equal(6);
              return committed.start({
                db: _db,
                softwareVersion: 'testSoftwareVersion',
                revisions: {
                  chainedTest: ['number']
                }
              }, function(err) {
                should.not.exist(err);
                return done();
              });
            });
          });
        });
      });
      it('should execute chains which operate within one single queue', function(done) {
        var bigT;
        bigT = committed.chain([
          committed.transaction('q', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ]);
        return committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return _db.collection('chainedTest', {
            strict: true
          }, function(err, collection) {
            should.not.exist(err);
            return collection.find({
              execute: 'all'
            }).toArray(function(err, docs) {
              docs.length.should.equal(6);
              return done();
            });
          });
        });
      });
      it('earlier queues used by a chain should not be blocked by the remainder of the chain', function(done) {
        var bigT, smallT, smallTDone;
        smallTDone = false;
        smallT = committed.transaction('q1', {}, [
          {
            name: "blockingMethod"
          }
        ]);
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q2', {}, [
            {
              name: "blockingMethod"
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ]);
        committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          smallTDone.should.equal(true);
          return done();
        });
        return committed.enqueue(smallT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return smallTDone = true;
        });
      });
      it('should be possible to execute chains that have GlobalLock transactions, one day', function(done) {
        var bigT;
        bigT = committed.chain([
          committed.transaction('q', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('GlobalLock', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ]);
        return committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return _db.collection('chainedTest', {
            strict: true
          }, function(err, collection) {
            should.not.exist(err);
            return collection.find({
              execute: 'all'
            }).toArray(function(err, docs) {
              docs.length.should.equal(6);
              return done();
            });
          });
        });
      });
      return it('should be possible to enqueue arrays of transactions with nulls in directly without manually forming a chain', function(done) {
        var bigT;
        bigT = [
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ]), null, committed.transaction('q3', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'chainedTest', {
                  execute: 'all'
                }
              ]
            }
          ])
        ];
        return committed.enqueue(bigT, function(err, status) {
          should.not.exist(err);
          status.should.equal('Committed');
          return _db.collection('chainedTest', {
            strict: true
          }, function(err, collection) {
            should.not.exist(err);
            return collection.find({
              execute: 'all'
            }).toArray(function(err, docs) {
              docs.length.should.equal(6);
              return done();
            });
          });
        });
      });
    });
    describe('restart', function() {
      beforeEach(function(done) {
        return _db.createCollection('transactions', function(err, collection) {
          return collection.remove({}, {
            w: 1
          }, function(err) {
            return _db.createCollection('restartTest', function(err, collection) {
              return collection.remove({}, {
                w: 1
              }, done);
            });
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      it('should execute any transactions left at Queued status', function(done) {
        var bigT, littleT, tinyT;
        tinyT = committed.transaction(null, {}, [
          {
            name: 'db.insert',
            "arguments": [
              'restartTest', {
                content: 'here'
              }
            ]
          }
        ]);
        littleT = committed.transaction('q1', {}, [
          {
            name: 'db.insert',
            "arguments": [
              'restartTest', {
                content: 'here'
              }
            ]
          }, {
            name: 'db.insert',
            "arguments": [
              'restartTest', {
                content: 'here'
              }
            ]
          }
        ]);
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }
          ])
        ]);
        tinyT.position = -1;
        littleT.position = 1;
        bigT.position = 2;
        return async.each([tinyT, littleT, bigT], function(item, itemDone) {
          return _db.collection('transactions', {
            strict: true
          }, function(err, collection) {
            return collection.insert(item, {
              w: 1,
              journal: true
            }, function(err, objects) {
              should.not.exist(err);
              return itemDone();
            });
          });
        }, function(err) {
          should.not.exist(err);
          return committed.start({
            db: _db,
            softwareVersion: 'testSoftwareVersion',
            revisions: {
              restartTest: ['number']
            }
          }, function(err) {
            should.not.exist(err);
            return _db.collection('restartTest', {
              strict: true
            }, function(err, collection) {
              should.not.exist(err);
              return collection.find({
                content: 'here'
              }).toArray(function(err, docs) {
                docs.length.should.equal(7);
                return done();
              });
            });
          });
        });
      });
      it('should rollback any transaction left at Processing status', function(done) {
        var bigT, doc1, doc2, littleT;
        littleT = committed.transaction('q1', {}, [
          {
            name: 'db.insert',
            "arguments": [
              'restartTest', {
                content: 'here'
              }
            ]
          }, {
            name: 'failMethod'
          }
        ], [
          {
            name: 'db.insertRollback',
            "arguments": [
              'restartTest', {
                content: 'here'
              }
            ]
          }, {
            name: 'db.pass'
          }
        ]);
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'failMethod'
            }
          ], [
            {
              name: 'db.insertRollback',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'db.pass'
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }
          ], [
            {
              name: 'db.insertRollback',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'db.pass'
            }
          ])
        ]);
        littleT.position = 1;
        bigT.position = 2;
        doc1 = {
          _id: ObjectID(),
          content: 'here'
        };
        doc2 = {
          _id: ObjectID(),
          content: 'here'
        };
        littleT.execution.state = [
          {
            ids: [doc1._id]
          }, {}
        ];
        littleT.execution.result = [true, false];
        littleT.status = 'Processing';
        bigT.execution.state = [
          {
            ids: [doc2._id]
          }, {}
        ];
        bigT.execution.result = [true, false];
        bigT.status = 'Processing';
        return async.each([['transactions', [littleT, bigT]], ['restartTest', [doc1, doc2]]], function(_arg, itemDone) {
          var what, where;
          where = _arg[0], what = _arg[1];
          return _db.collection(where, {
            strict: true
          }, function(err, collection) {
            return collection.insert(what, {
              w: 1,
              journal: true
            }, function(err, objects) {
              should.not.exist(err);
              return itemDone();
            });
          });
        }, function(err) {
          should.not.exist(err);
          return committed.start({
            db: _db,
            softwareVersion: 'testSoftwareVersion',
            revisions: {
              restartTest: ['number']
            }
          }, function(err) {
            should.not.exist(err);
            return _db.collection('restartTest', {
              strict: true
            }, function(err, collection) {
              should.not.exist(err);
              return collection.find({
                content: 'here'
              }).toArray(function(err, docs) {
                docs.length.should.equal(0);
                return done();
              });
            });
          });
        });
      });
      return it('should restart execution of a stalled chain', function(done) {
        var bigT;
        bigT = committed.chain([
          committed.transaction('q1', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }
          ]), committed.transaction('q2', {}, [
            {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }, {
              name: 'db.insert',
              "arguments": [
                'restartTest', {
                  content: 'here'
                }
              ]
            }
          ])
        ]);
        bigT.status = 'Committed';
        bigT.position = 1;
        return _db.collection('transactions', {
          strict: true
        }, function(err, collection) {
          return collection.insert(bigT, {
            w: 1,
            journal: true
          }, function(err, objects) {
            should.not.exist(err);
            return committed.start({
              db: _db,
              softwareVersion: 'testSoftwareVersion',
              revisions: {
                restartTest: ['number']
              }
            }, function(err) {
              should.not.exist(err);
              return _db.collection('restartTest', {
                strict: true
              }, function(err, collection) {
                should.not.exist(err);
                return collection.find({
                  content: 'here'
                }).toArray(function(err, docs) {
                  docs.length.should.equal(2);
                  return done();
                });
              });
            });
          });
        });
      });
    });
    describe('function enqueuement', function() {
      beforeEach(function(done) {
        return committed.start({
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            functionTest: ['number']
          }
        }, function(err) {
          should.not.exist(err);
          return _db.createCollection('functionTest', function(err, collection) {
            return collection.remove({}, {
              w: 1
            }, done);
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      it('should execute writer functions which return a status string', function(done) {
        var myWriter;
        myWriter = committed.writer('myQueue', function(writerDone) {
          return _db.collection('functionTest', function(err, collection) {
            if (err != null) {
              return writerDone(err, null);
            }
            return collection.insert({
              mycontent: 'is good'
            }, {
              w: 1,
              journal: true
            }, function(err, objects) {
              if (err != null) {
                return writerDone(err, null);
              }
              return writerDone(null, 'Committed');
            });
          });
        });
        return committed.enqueue(myWriter, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          return _db.collection('functionTest', function(err, collection) {
            return collection.count({
              mycontent: 'is good'
            }, function(err, count) {
              count.should.equal(1);
              return done();
            });
          });
        });
      });
      it('should execute writer functions which produce a transaction object, and save it to the transaction collection', function(done) {
        var docs, i, transaction;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        transaction = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['functionTest', docs]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          var myWriter;
          should.not.exist(err);
          status.should.be.string('Committed');
          myWriter = committed.writer('a_queue', function(done) {
            return setTimeout(function() {
              var d, ids, update;
              ids = (function() {
                var _i, _len, _results;
                _results = [];
                for (_i = 0, _len = docs.length; _i < _len; _i++) {
                  d = docs[_i];
                  _results.push(d._id);
                }
                return _results;
              })();
              update = committed.transaction("a_queue", {}, [
                {
                  name: 'db.updateOneOp',
                  "arguments": [
                    'functionTest', {
                      _id: {
                        $in: ids
                      },
                      i: {
                        $gt: 2
                      }
                    }, {
                      $inc: {
                        j: 1
                      }
                    }
                  ]
                }
              ]);
              return done(null, update);
            }, 500);
          });
          return _db.collection('transactions', function(err, transactions) {
            return transactions.count(function(err, initialCount) {
              should.not.exist(err);
              return committed.enqueue(myWriter, function(err, status) {
                should.not.exist(err);
                status.should.be.string('Committed');
                return _db.collection('functionTest', function(err, functionTestCollection) {
                  return functionTestCollection.find({
                    j: 1
                  }).toArray(function(err, docsInDB) {
                    should.not.exist(err);
                    docsInDB.length.should.equal(1);
                    docsInDB[0].revision.number.should.equal(2);
                    return done();
                  });
                });
              });
            });
          });
        });
      });
      it('should execute writers with global lock', function(done) {
        var myWriter;
        myWriter = committed.writer('GlobalLock', function(writerDone) {
          return _db.collection('functionTest', function(err, collection) {
            if (err != null) {
              return writerDone(err, null);
            }
            return collection.insert({
              mycontent: 'is good'
            }, {
              w: 1,
              journal: true
            }, function(err, objects) {
              if (err != null) {
                return writerDone(err, null);
              }
              return writerDone(null, 'Committed');
            });
          });
        });
        return committed.withGlobalLock(myWriter, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          return _db.collection('functionTest', function(err, collection) {
            return collection.count({
              mycontent: 'is good'
            }, function(err, count) {
              count.should.equal(1);
              return done();
            });
          });
        });
      });
      it('should execute writer functions which produce a transaction chain', function(done) {
        var doc, insert, myChain, updateWriter;
        doc = {
          i: 1
        };
        insert = committed.transaction("queue", {}, [
          {
            name: 'db.insert',
            "arguments": ['functionTest', doc]
          }
        ]);
        myChain = null;
        updateWriter = committed.writer('queue', function(done) {
          return setTimeout(function() {
            var update, update_2, update_3;
            update = committed.transaction("queue", {}, [
              {
                name: 'db.updateOneOp',
                "arguments": [
                  'functionTest', {
                    _id: doc._id,
                    i: 1
                  }, {
                    $inc: {
                      i: 1
                    }
                  }
                ]
              }
            ]);
            update_2 = committed.transaction("queue_2", {}, [
              {
                name: 'db.updateOneOp',
                "arguments": [
                  'functionTest', {
                    _id: doc._id,
                    i: 2
                  }, {
                    $inc: {
                      i: 1
                    }
                  }
                ]
              }
            ]);
            update_3 = committed.transaction("queue_3", {}, [
              {
                name: 'db.updateOneOp',
                "arguments": [
                  'functionTest', {
                    _id: doc._id,
                    i: 3
                  }, {
                    $inc: {
                      i: 1
                    }
                  }
                ]
              }
            ]);
            myChain = committed.chain([update, update_2, update_3]);
            return done(null, myChain);
          }, 500);
        });
        committed.enqueue(insert, function(err, status) {
          should.not.exist(err);
          return status.should.be.string('Committed');
        });
        return committed.enqueue(updateWriter, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          return _db.collection('functionTest', function(err, functionTestCollection) {
            return functionTestCollection.count({
              i: 4
            }, function(err, count) {
              should.not.exist(err);
              count.should.equal(1);
              return done();
            });
          });
        });
      });
      it('cannot build chains made up of writers', function(done) {
        var doc, inWriter, makeChain, upWriter;
        doc = {
          i: 1
        };
        inWriter = committed.writer('queue', function(done) {
          var insert;
          insert = committed.transaction("queue", {}, [
            {
              name: 'db.insert',
              "arguments": ['functionTest', doc]
            }
          ]);
          return done(null, insert);
        });
        upWriter = committed.writer('queue_2', function(done) {
          var update;
          update = committed.transaction("queue_2", {}, [
            {
              name: 'db.updateOneOp',
              "arguments": [
                'functionTest', {
                  _id: doc._id,
                  i: 1
                }, {
                  $inc: {
                    i: 1
                  }
                }
              ]
            }
          ]);
          return done(null, update);
        });
        makeChain = function() {
          return committed.chain([inWriter, upWriter]);
        };
        makeChain.should["throw"](Error);
        return done();
      });
      it('should execute reader functions', function(done) {
        var myReader;
        myReader = committed.reader('queue', function(readerDone) {
          return setTimeout(function() {
            return readerDone(null, 1, 2, 3);
          });
        });
        return committed.enqueue(myReader, function(err, a, b, c) {
          should.not.exist(err);
          a.should.equal(1);
          b.should.equal(2);
          c.should.equal(3);
          return done();
        });
      });
      it('should execute readers with global lock', function(done) {
        var myReader;
        myReader = committed.reader('GlobalLock', function(readerDone) {
          return setTimeout(function() {
            return readerDone(null, 1, 2, 3);
          });
        });
        return committed.withGlobalLock(myReader, function(err, a, b, c) {
          should.not.exist(err);
          a.should.equal(1);
          b.should.equal(2);
          c.should.equal(3);
          return done();
        });
      });
      it('should return an error when a function produces a transaction whose queue fails to match that of the function', function(done) {
        var badWriter;
        badWriter = committed.writer('queue', function(done) {
          var insert;
          insert = committed.transaction("wrong_queue", {}, [
            {
              name: 'db.insert',
              "arguments": [
                'functionTest', {
                  i: 1
                }
              ]
            }
          ]);
          return done(null, insert);
        });
        return committed.enqueue(badWriter, function(err, status) {
          should.exist(err);
          should.not.exist(status);
          return done();
        });
      });
      it('a writer should pass through an error when the function produces one', function(done) {
        var badWriter;
        badWriter = committed.writer('queue', function(done) {
          return done(new Error("this is my error"), null);
        });
        return committed.enqueue(badWriter, function(err, status) {
          should.exist(err);
          err.message.should.be.string("this is my error");
          should.not.exist(status);
          return done();
        });
      });
      it('a writer should pass through an error when the function produces one, even if a transaction is returned', function(done) {
        var badWriter;
        badWriter = committed.writer('queue', function(done) {
          var t;
          t = committed.transaction("queue", {}, [
            {
              name: 'db.insert',
              "arguments": [
                'functionTest', {
                  foo: 1
                }
              ]
            }
          ]);
          return done(new Error("this is my error"), t);
        });
        return committed.enqueue(badWriter, function(err, status) {
          should.exist(err);
          err.message.should.be.string("this is my error");
          should.not.exist(status);
          return done();
        });
      });
      it('a writer should pass through an error when the function produces one, even if an empty array is returned (bad chain)', function(done) {
        var badWriter;
        badWriter = committed.writer('queue', function(done) {
          return done(new Error("this is my error"), []);
        });
        return committed.enqueue(badWriter, function(err, status) {
          should.exist(err);
          err.message.should.be.string("this is my error");
          should.not.exist(status);
          return done();
        });
      });
      it('a reader should pass through an error when its function produces one', function(done) {
        var myReader;
        myReader = committed.reader('queue', function(readerDone) {
          return readerDone(new Error("my error"));
        });
        return committed.enqueue(myReader, function(err) {
          should.exist(err);
          err.message.should.be.string('my error');
          return done();
        });
      });
      return it('a writer should be processed immediately', function(done) {
        var myWriter;
        myWriter = committed.writer(null, function(writerDone) {
          return _db.collection('functionTest', function(err, collection) {
            if (err != null) {
              return writerDone(err, null);
            }
            return collection.insert({
              immediatestuff: 'is good'
            }, {
              w: 1,
              journal: true
            }, function(err, objects) {
              if (err != null) {
                return writerDone(err, null);
              }
              return writerDone(null, 'Committed');
            });
          });
        });
        return committed.immediately(myWriter, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          return _db.collection('functionTest', function(err, collection) {
            return collection.count({
              immediatestuff: 'is good'
            }, function(err, count) {
              count.should.equal(1);
              return done();
            });
          });
        });
      });
    });
    describe('returning results', function(done) {
      beforeEach(function(done) {
        var config;
        config = {
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            resultsTest: ['number']
          }
        };
        return committed.start(config, function(err) {
          should.not.exist(err);
          return _db.createCollection('resultsTest', function(err, collection) {
            return collection.remove({}, {
              w: 1
            }, done);
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      it('should execute an instruction in a single transaction and accumulate the result it yields', function(done) {
        var docs, i, insert;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        insert = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['resultsTest', docs]
          }
        ]);
        return committed.enqueue(insert, function(err, status) {
          var d, ids, update;
          should.not.exist(err);
          status.should.be.string('Committed');
          ids = (function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = docs.length; _i < _len; _i++) {
              d = docs[_i];
              _results.push(d._id);
            }
            return _results;
          })();
          update = committed.transaction("test", {}, [
            {
              name: 'db.updateManyOp',
              "arguments": [
                'resultsTest', {
                  _id: {
                    $in: ids
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }
          ]);
          return committed.enqueue(update, function(err, status, updated) {
            should.not.exist(err);
            status.should.be.string('Committed');
            updated.should.equal(3);
            return done();
          });
        });
      });
      it('should execute multiple instructions a single transaction and accumulate any results they yield', function(done) {
        var docs, i, update;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        update = committed.transaction("test", {}, [
          {
            name: 'db.insert',
            "arguments": ['resultsTest', docs]
          }, {
            name: 'db.updateManyOp',
            "arguments": [
              'resultsTest', {
                i: {
                  $gte: 1
                }
              }, {
                $inc: {
                  j: 1
                }
              }
            ]
          }, {
            name: 'db.updateManyOp',
            "arguments": [
              'resultsTest', {
                i: {
                  $gte: 2
                }
              }, {
                $inc: {
                  j: 1
                }
              }
            ]
          }, {
            name: 'db.updateManyOp',
            "arguments": [
              'resultsTest', {
                i: {
                  $gte: 3
                }
              }, {
                $inc: {
                  j: 1
                }
              }
            ]
          }
        ]);
        return committed.enqueue(update, function(err, status, a, b, c) {
          should.not.exist(err);
          status.should.be.string('Committed');
          a.should.equal(3);
          b.should.equal(2);
          c.should.equal(1);
          return done();
        });
      });
      it('should execute multiple instructions and a writer and accumulate any results they yield', function(done) {
        var docs, i, writer;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        writer = committed.writer("test", function(writerDone) {
          var update;
          update = committed.transaction("test", {}, [
            {
              name: 'db.insert',
              "arguments": ['resultsTest', docs]
            }, {
              name: 'db.updateManyOp',
              "arguments": [
                'resultsTest', {
                  i: {
                    $gte: 1
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }, {
              name: 'db.updateManyOp',
              "arguments": [
                'resultsTest', {
                  i: {
                    $gte: 2
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }, {
              name: 'db.updateManyOp',
              "arguments": [
                'resultsTest', {
                  i: {
                    $gte: 3
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }
          ]);
          return writerDone(null, update, 'my writer result');
        });
        return committed.enqueue(writer, function(err, status, s, a, b, c) {
          should.not.exist(err);
          status.should.be.string('Committed');
          s.should.be.string('my writer result');
          a.should.equal(3);
          b.should.equal(2);
          c.should.equal(1);
          return done();
        });
      });
      return it('should execute a chain produced by a writer and accumulate any results', function(done) {
        var docs, i, writer;
        docs = (function() {
          var _i, _len, _ref, _results;
          _ref = [1, 2, 3];
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            i = _ref[_i];
            _results.push({
              i: i
            });
          }
          return _results;
        })();
        writer = committed.writer("test", function(writerDone) {
          var chain, update1, update2;
          update1 = committed.transaction("test", {}, [
            {
              name: 'db.insert',
              "arguments": ['resultsTest', docs]
            }, {
              name: 'db.updateManyOp',
              "arguments": [
                'resultsTest', {
                  i: {
                    $gte: 1
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }
          ]);
          update2 = committed.transaction("test2", {}, [
            {
              name: 'db.updateManyOp',
              "arguments": [
                'resultsTest', {
                  i: {
                    $gte: 2
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }, {
              name: 'db.updateOneOp',
              "arguments": [
                'resultsTest', {
                  i: {
                    $gte: 3
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }, {
              name: 'db.updateManyOp',
              "arguments": [
                'resultsTest', {
                  i: {
                    $gte: 3
                  }
                }, {
                  $inc: {
                    j: 1
                  }
                }
              ]
            }
          ]);
          chain = committed.chain([update1, update2]);
          return writerDone(null, chain, 'my writer result');
        });
        return committed.enqueue(writer, function(err, status, s, a, b, c) {
          should.not.exist(err);
          status.should.be.string('Committed');
          s.should.be.string('my writer result');
          a.should.equal(3);
          b.should.equal(2);
          c.should.equal(1);
          return done();
        });
      });
    });
    return describe('regex queue name checks', function() {
      beforeEach(function(done) {
        var config;
        config = {
          db: _db,
          softwareVersion: 'testSoftwareVersion',
          revisions: {
            regexTest: ['number']
          },
          queueNameRegex: /^foo$|^bar$/
        };
        return committed.start(config, function(err) {
          should.not.exist(err);
          return _db.createCollection('regexTest', function(err, collection) {
            return collection.remove({}, {
              w: 1
            }, done);
          });
        });
      });
      afterEach(function(done) {
        return committed.stop(done);
      });
      it('should run a transaction which passes the queue name regex', function(done) {
        var doc, transaction;
        doc = {
          t: 't',
          q: 'q'
        };
        transaction = committed.transaction("foo", {}, [
          {
            name: 'db.insert',
            "arguments": ['regexTest', doc]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          should.not.exist(err);
          status.should.be.string('Committed');
          should.exist(doc._id);
          return _db.collection('regexTest', function(err, collection) {
            should.not.exist(err);
            return collection.findOne({
              _id: doc._id
            }, function(err, docInDB) {
              should.not.exist(err);
              doc.t.should.equal(docInDB.t);
              doc.q.should.equal(docInDB.q);
              return done();
            });
          });
        });
      });
      return it('should fail to run a transaction which doesnt pass the regex', function(done) {
        var doc, transaction;
        doc = {
          t: 't',
          q: 'q'
        };
        transaction = committed.transaction("baz", {}, [
          {
            name: 'db.insert',
            "arguments": ['regexTest', doc]
          }
        ]);
        return committed.enqueue(transaction, function(err, status) {
          err.toString().should.equal("Error: transaction.queue name baz does not match required regex /^foo$|^bar$/");
          return done();
        });
      });
    });
  });

}).call(this);
